/* --------------------------------------------------------- */
/*	SCI Interrupt test										 */
/* ========================================================= */
/*	(C)2004 t.hara											 */
/* --------------------------------------------------------- */

#include "cpu.h"

/*	Virtual Interrupt Table	*/
#define	VINT_TBL	0xFFEF10
#define	LinkERI1	(VINT_TBL + 4*56)
#define	LinkRXI1	(VINT_TBL + 4*57)

#define	SCR1		0xFFFFBA
#define	SSR1	 	0xFFFFBC
#define	RDR1		0xFFFFBD

#define	LCD_DATA	0xFFFFD3

typedef struct {
	unsigned char b7:1;
	unsigned char b6:1;
	unsigned char b5:1;
	unsigned char b4:1;
	unsigned char b3:1;
	unsigned char b2:1;
	unsigned char b1:1;
	unsigned char b0:1;
} BIT;

#define	POKE8( Adr, Data )	( *(volatile unsigned char*)(Adr) = (Data) )
#define	PEEK8( Adr )		( *(volatile unsigned char*)(Adr) )
#define	POKE32( Adr, Data )	( *(volatile unsigned long*)(Adr) = (Data) )

/* --------------------------------------------------------- */
static volatile int fBuffering = 0;
static unsigned char Fifo[ 256 ];
static unsigned char FifoReadPtr = 0;
static unsigned char FifoWritePtr = 0;

/* --------------------------------------------------------- */
static void WaitLcd( void ) {
	volatile int i;

	for( i = 0; i < 170; i++ );
}

/* --------------------------------------------------------- */
static void LcdPutc( char c ) {

	POKE8( LCD_DATA, ((c >> 4) & 0x0F) | 0x30 );
	WaitLcd();
	POKE8( LCD_DATA, ((c >> 4) & 0x0F) | 0x10 );
	WaitLcd();
	POKE8( LCD_DATA, (c & 0x0F) | 0x30 );
	WaitLcd();
	POKE8( LCD_DATA, (c & 0x0F) | 0x10 );
	WaitLcd();
}

/* --------------------------------------------------------- */
static void LcdCommand( char c ) {

	POKE8( LCD_DATA, ((c >> 4) & 0x0F) | 0x20 );
	WaitLcd();
	POKE8( LCD_DATA, ((c >> 4) & 0x0F) | 0x00 );
	WaitLcd();
	POKE8( LCD_DATA, (c & 0x0F) | 0x20 );
	WaitLcd();
	POKE8( LCD_DATA, (c & 0x0F) | 0x00 );
	WaitLcd();
}

/* --------------------------------------------------------- */
static void LcdPuts( const char *pStr ) {

	while( *pStr ) {
		LcdPutc( *pStr );
		++pStr;
	}
}

/* --------------------------------------------------------- */
static void Sci1ErrorInterrupt( void ) {

	POKE8( SSR1, PEEK8( SSR1 ) & 0xCF );	/* clear error */
}

/* --------------------------------------------------------- */
static void Sci1ReceiveInterrupt( void ) {
	unsigned char c;

	POKE8( SSR1, PEEK8( SSR1 ) & 0xBF );
	if( !fBuffering ) return;
	c = PEEK8( RDR1 );
	if( c < ' ' ) {
		fBuffering = 0;
		return;
	}
	if( (FifoWritePtr+1) == FifoReadPtr ) {
		fBuffering = 0;
		return;
	}
	Fifo[ FifoWritePtr++ ] = c;
}

/* --------------------------------------------------------- */
static void SetUpSciInterrupt( void ) {
	volatile int i;

	/* setup interrupt vector */
	POKE32( LinkERI1, (unsigned long) Sci1ErrorInterrupt );
	POKE32( LinkRXI1, (unsigned long) Sci1ReceiveInterrupt );

	/* enable SCI1 interrupt mode */
	POKE8( SCR1, 0x00 );	/* U~ */
	POKE8( SCR1, 0x40 );	/* M荞݋𗧂Ă */
	for( i = 0; i < 280; i++ );
	POKE8( SCR1, 0x70 );	/* TE, RE 𗧂Ă */
	POKE8( SSR1, 0x84 );
}

/* --------------------------------------------------------- */
void Wait( void ) {
	volatile int i;

	for( i = 0; i < 1000; i++ );
}

/* --------------------------------------------------------- */
void main( void ) {

	LcdCommand( 0x01 );
	Wait();
	LcdPuts( "SCI INT TEST" );
	SetUpSciInterrupt();
	for( ;; ) {
		fBuffering = 1;
		EnableInterrupt();
		while( fBuffering );
		DisableInterrupt();
		LcdCommand( 0x80 );
		Wait();
		while( FifoReadPtr != FifoWritePtr ) {
			LcdPutc( Fifo[ FifoReadPtr++ ] );
		}
	}
}
